@@ -17,6 +17,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
+
 #include <gtk/gtk.h>
 #include <NvCtrlAttributes.h>
 #include <string.h>
@@ -45,21 +46,25 @@ static void update_device_info(CtkDisplayDevice *ctk_object);
 
 static void display_device_setup(CtkDisplayDevice *ctk_object);
 
-static void enabled_displays_received(GtkObject *object, gpointer arg1,
+static void enabled_displays_received(GObject *object, CtrlEvent *event,
                                       gpointer user_data);
 
-static void callback_link_changed(GtkObject *object, gpointer arg1,
+static void callback_link_changed(GObject *object, CtrlEvent *event,
                                   gpointer user_data);
 
-static void callback_refresh_rate_changed(GtkObject *object, gpointer arg1,
+static void callback_refresh_rate_changed(GObject *object, CtrlEvent *event,
                                           gpointer user_data);
 
+static gboolean update_guid_info(InfoEntry *entry);
 static gboolean update_tv_encoder_info(InfoEntry *entry);
 static gboolean update_chip_info(InfoEntry *entry);
 static gboolean update_signal_info(InfoEntry *entry);
 static gboolean update_link_info(InfoEntry *entry);
-static gboolean update_native_resolution(InfoEntry *entry);
 static gboolean update_refresh_rate(InfoEntry *entry);
+static gboolean update_connector_type_info(InfoEntry *entry);
+static gboolean update_multistream_info(InfoEntry *entry);
+static gboolean update_audio_info(InfoEntry *entry);
+static gboolean update_vrr_type_info(InfoEntry *entry);
 
 static gboolean register_link_events(InfoEntry *entry);
 static gboolean unregister_link_events(InfoEntry *entry);
@@ -78,6 +83,9 @@ static const char *__info_help =
 "This section describes basic information about the connection to the display "
 "device.";
 
+static const char*__guid_help =
+"The Global Unique Identifier for the display port display device.";
+
 static const char* __tv_encoder_name_help =
 "The TV Encoder name displays the name of the TV Encoder.";
 
@@ -95,15 +103,23 @@ static const char *__info_signal_help =
 "Report whether the flat panel is driven by an LVDS, TMDS, or DisplayPort "
 "signal.";
 
-static const char * __native_res_help =
-"The Native Resolution is the width and height in pixels that the display "
-"device uses to display the image.  All other resolutions must be scaled "
-"to this resolution by the GPU and/or the device's built-in scaler.";
-
 static const char * __refresh_rate_help =
 "The refresh rate displays the rate at which the screen is currently "
 "refreshing the image.";
 
+static const char * __connector_type_help =
+"Report the connector type that the DisplayPort display is using.";
+
+static const char * __multistream_help =
+"Report whether the configured DisplayPort display supports multistream.";
+
+static const char * __audio_help =
+"Report whether the configured DisplayPort display is capable of playing audio.";
+
+static const char * __vrr_type_help =
+"Report whether the configured display supports G-SYNC, G-SYNC Compatible, or "
+"neither.";
+
 typedef gboolean (*InfoEntryFunc)(InfoEntry *entry);
 
 typedef struct {
@@ -116,7 +132,14 @@ typedef struct {
 
 static InfoEntryData __info_entry_data[] = {
     {
-        "TV Encoder:",
+        "GUID",
+        &__guid_help,
+        update_guid_info,
+        NULL,
+        NULL,
+    },
+    {
+        "TV Encoder",
         &__tv_encoder_name_help,
         update_tv_encoder_info,
         NULL,
@@ -143,13 +166,6 @@ static InfoEntryData __info_entry_data[] = {
         register_link_events,
         unregister_link_events,
     },
-    {
-        "Native Resolution",
-        &__native_res_help,
-        update_native_resolution,
-        NULL,
-        NULL,
-    },
     {
         "Refresh Rate",
         &__refresh_rate_help,
@@ -157,6 +173,34 @@ static InfoEntryData __info_entry_data[] = {
         register_refresh_rate_events,
         unregister_refresh_rate_events,
     },
+    {
+        "DisplayPort Connector Type",
+        &__connector_type_help,
+        update_connector_type_info,
+        NULL,
+        NULL,
+    },
+    {
+        "DisplayPort Multistream Available",
+        &__multistream_help,
+        update_multistream_info,
+        NULL,
+        NULL,
+    },
+    {
+        "DisplayPort Audio Available",
+        &__audio_help,
+        update_audio_info,
+        NULL,
+        NULL,
+    },
+    {
+        "G-SYNC Mode Available",
+        &__vrr_type_help,
+        update_vrr_type_info,
+        NULL,
+        NULL,
+    },
 };
 
 GType ctk_display_device_get_type(void)
@@ -225,7 +269,7 @@ static void ctk_display_device_finalize(
  * ctk_display_device_new() - constructor for the dissplay device page.
  */
 
-GtkWidget* ctk_display_device_new(NvCtrlAttributeHandle *handle,
+GtkWidget* ctk_display_device_new(CtrlTarget *ctrl_target,
                                   CtkConfig *ctk_config,
                                   CtkEvent *ctk_event,
                                   CtkEvent *ctk_event_gpu,
@@ -252,7 +296,7 @@ GtkWidget* ctk_display_device_new(NvCtrlAttributeHandle *handle,
     if (!object) return NULL;
 
     ctk_object = CTK_DISPLAY_DEVICE(object);
-    ctk_object->handle = handle;
+    ctk_object->ctrl_target = ctrl_target;
     ctk_object->ctk_event = ctk_event;
     ctk_object->ctk_event_gpu = ctk_event_gpu;
     ctk_object->ctk_config = ctk_config;
@@ -353,7 +397,7 @@ GtkWidget* ctk_display_device_new(NvCtrlAttributeHandle *handle,
 
     /* pack the EDID button */
 
-    ctk_object->edid = ctk_edid_new(ctk_object->handle,
+    ctk_object->edid = ctk_edid_new(ctrl_target,
                                     ctk_object->ctk_config,
                                     ctk_object->ctk_event,
                                     ctk_object->name);
@@ -387,7 +431,7 @@ GtkWidget* ctk_display_device_new(NvCtrlAttributeHandle *handle,
     /* pack the color controls */
 
     ctk_object->color_controls =
-        ctk_color_controls_new(handle, ctk_config, ctk_event,
+        ctk_color_controls_new(ctrl_target, ctk_config, ctk_event,
                                ctk_object->reset_button, name);
 
     if (ctk_object->color_controls) {
@@ -398,7 +442,7 @@ GtkWidget* ctk_display_device_new(NvCtrlAttributeHandle *handle,
     /* pack the dithering controls */
 
     ctk_object->dithering_controls =
-        ctk_dithering_controls_new(handle, ctk_config, ctk_event,
+        ctk_dithering_controls_new(ctrl_target, ctk_config, ctk_event,
                                    ctk_object->reset_button, name);
 
     if (ctk_object->dithering_controls) {
@@ -409,7 +453,7 @@ GtkWidget* ctk_display_device_new(NvCtrlAttributeHandle *handle,
     /* pack the image sliders */
 
     ctk_object->image_sliders =
-        ctk_image_sliders_new(handle, ctk_config, ctk_event,
+        ctk_image_sliders_new(ctrl_target, ctk_config, ctk_event,
                               ctk_object->reset_button, name);
     if (ctk_object->image_sliders) {
         gtk_box_pack_start(GTK_BOX(nbox), ctk_object->image_sliders,
@@ -425,13 +469,19 @@ GtkWidget* ctk_display_device_new(NvCtrlAttributeHandle *handle,
                                  gtk_label_new("Controls"));
     }
 
+    /*
+     * Show all widgets on this page so far. After this, the color correction
+     * tab and other widgets can control their own visibility.
+     */
+
+    gtk_widget_show_all(GTK_WIDGET(object));
+
     /* add the color correction tab if RandR is available */
 
     add_color_correction_tab(ctk_object, ctk_config, ctk_event, notebook, p);
 
     /* Update the GUI */
 
-    gtk_widget_show_all(GTK_WIDGET(object));
     display_device_setup(ctk_object);
 
     /* Listen to events */
@@ -497,7 +547,7 @@ GtkTextBuffer *ctk_display_device_create_help(GtkTextTagTable *table,
 {
     GtkTextIter i;
     GtkTextBuffer *b;
-    GtkTooltipsData *td;
+    gchar *tip_text;
     int j;
 
     b = gtk_text_buffer_new(table);
@@ -534,8 +584,9 @@ GtkTextBuffer *ctk_display_device_create_help(GtkTextTagTable *table,
         ctk_color_correction_tab_help(b, &i, "X Server Color Correction", TRUE);
     }
 
-    td = gtk_tooltips_data_get(GTK_WIDGET(ctk_object->reset_button));
-    ctk_help_reset_hardware_defaults(b, &i, td->tip_text);
+    tip_text = ctk_widget_get_tooltip_text(GTK_WIDGET(ctk_object->reset_button));
+    ctk_help_reset_hardware_defaults(b, &i, tip_text);
+    g_free(tip_text);
 
     ctk_help_finish(b);
 
@@ -544,21 +595,47 @@ GtkTextBuffer *ctk_display_device_create_help(GtkTextTagTable *table,
 } /* ctk_display_device_create_help() */
 
 
-static gboolean update_tv_encoder_info(InfoEntry *entry)
+
+static gboolean update_guid_info(InfoEntry *entry)
 {
     CtkDisplayDevice *ctk_object = entry->ctk_object;
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
     ReturnStatus ret;
     char *str;
 
-    ret = NvCtrlGetStringDisplayAttribute(ctk_object->handle, 0,
-                                          NV_CTRL_STRING_TV_ENCODER_NAME, &str);
+    ret = NvCtrlGetStringAttribute(ctrl_target,
+                                   NV_CTRL_STRING_DISPLAY_NAME_DP_GUID,
+                                   &str);
     if (ret != NvCtrlSuccess) {
         return FALSE;
     }
 
     gtk_label_set_text(GTK_LABEL(entry->txt), str);
 
-    XFree(str);
+    free(str);
+
+    return TRUE;
+}
+
+
+
+static gboolean update_tv_encoder_info(InfoEntry *entry)
+{
+    CtkDisplayDevice *ctk_object = entry->ctk_object;
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
+    ReturnStatus ret;
+    char *str;
+
+    ret = NvCtrlGetStringAttribute(ctrl_target,
+                                   NV_CTRL_STRING_TV_ENCODER_NAME,
+                                   &str);
+    if (ret != NvCtrlSuccess) {
+        return FALSE;
+    }
+
+    gtk_label_set_text(GTK_LABEL(entry->txt), str);
+
+    free(str);
 
     return TRUE;
 }
@@ -568,11 +645,12 @@ static gboolean update_tv_encoder_info(InfoEntry *entry)
 static gboolean update_chip_info(InfoEntry *entry)
 {
     CtkDisplayDevice *ctk_object = entry->ctk_object;
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
     ReturnStatus ret;
     gint val;
     const gchar *str;
 
-    ret = NvCtrlGetAttribute(ctk_object->handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_FLATPANEL_CHIP_LOCATION, &val);
     if (ret != NvCtrlSuccess) {
         return FALSE;
@@ -600,12 +678,12 @@ static gboolean update_chip_info(InfoEntry *entry)
 static gboolean update_signal_info(InfoEntry *entry)
 {
     CtkDisplayDevice *ctk_object = entry->ctk_object;
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
     ReturnStatus ret;
     gint val;
     const char *str;
 
-    ret = NvCtrlGetAttribute(ctk_object->handle, NV_CTRL_FLATPANEL_SIGNAL,
-                             &val);
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_FLATPANEL_SIGNAL, &val);
     if (ret != NvCtrlSuccess) {
         return FALSE;
     }
@@ -639,12 +717,13 @@ static gboolean update_signal_info(InfoEntry *entry)
 static gboolean update_link_info(InfoEntry *entry)
 {
     CtkDisplayDevice *ctk_object = entry->ctk_object;
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
     ReturnStatus ret;
     gint val;
     const char *link;
     char tmp[32];
 
-    ret = NvCtrlGetAttribute(ctk_object->handle, NV_CTRL_FLATPANEL_LINK, &val);
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_FLATPANEL_LINK, &val);
     if (ret != NvCtrlSuccess) {
         return FALSE;
     }
@@ -654,8 +733,8 @@ static gboolean update_link_info(InfoEntry *entry)
 
         lanes = val + 1;
 
-        ret = NvCtrlGetAttribute(ctk_object->handle,
-                                 NV_CTRL_DISPLAYPORT_LINK_RATE, &val);
+        ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_DISPLAYPORT_LINK_RATE,
+                                 &val);
         if ((ret == NvCtrlSuccess) &&
             (val == NV_CTRL_DISPLAYPORT_LINK_RATE_DISABLED)) {
             link = "Disabled";
@@ -694,44 +773,137 @@ static gboolean update_link_info(InfoEntry *entry)
 }
 
 
-
-static gboolean update_native_resolution(InfoEntry *entry)
+static gboolean update_connector_type_info(InfoEntry *entry)
 {
     CtkDisplayDevice *ctk_object = entry->ctk_object;
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
     ReturnStatus ret;
     gint val;
-    char *str;
+    const char *str;
 
-    ret = NvCtrlGetAttribute(ctk_object->handle,
-                             NV_CTRL_FLATPANEL_NATIVE_RESOLUTION, &val);
+    ret = NvCtrlGetAttribute(ctrl_target,
+                             NV_CTRL_DISPLAYPORT_CONNECTOR_TYPE, &val);
     if (ret != NvCtrlSuccess) {
         return FALSE;
     }
 
-    str = g_strdup_printf("%dx%d", (val >> 16), (val & 0xFFFF));
+    switch (val) {
+    case NV_CTRL_DISPLAYPORT_CONNECTOR_TYPE_DISPLAYPORT:
+        str = "DisplayPort";
+        break;
+    case NV_CTRL_DISPLAYPORT_CONNECTOR_TYPE_HDMI:
+        str = "HDMI";
+        break;
+    case NV_CTRL_DISPLAYPORT_CONNECTOR_TYPE_DVI:
+        str = "DVI";
+        break;
+    case NV_CTRL_DISPLAYPORT_CONNECTOR_TYPE_VGA:
+        str = "VGA";
+        break;
+    default:
+    case NV_CTRL_DISPLAYPORT_CONNECTOR_TYPE_UNKNOWN:
+        str = "Unknown";
+        break;
+    }
+
+    gtk_label_set_text(GTK_LABEL(entry->txt), str);
+
+    return TRUE;
+}
+
+
+static gboolean update_multistream_info(InfoEntry *entry)
+{
+    CtkDisplayDevice *ctk_object = entry->ctk_object;
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
+    ReturnStatus ret;
+    gint val;
+
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_DISPLAYPORT_IS_MULTISTREAM,
+                             &val);
+    if (ret != NvCtrlSuccess) {
+        return FALSE;
+    }
+
+    gtk_label_set_text(GTK_LABEL(entry->txt), val ? "Yes": "No");
+
+    return TRUE;
+}
+
+static gboolean update_audio_info(InfoEntry *entry)
+{
+    CtkDisplayDevice *ctk_object = entry->ctk_object;
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
+    ReturnStatus ret;
+    gint val;
+
+    ret = NvCtrlGetAttribute(ctrl_target,
+                             NV_CTRL_DISPLAYPORT_SINK_IS_AUDIO_CAPABLE,
+                             &val);
+    if (ret != NvCtrlSuccess) {
+        return FALSE;
+    }
+
+    gtk_label_set_text(GTK_LABEL(entry->txt), val ? "Yes": "No");
+
+    return TRUE;
+}
+
+static gboolean update_vrr_type_info(InfoEntry *entry)
+{
+    CtkDisplayDevice *ctk_object = entry->ctk_object;
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
+    ReturnStatus ret;
+    gint val;
+    const char *str;
+
+    ret = NvCtrlGetAttribute(ctrl_target,
+                             NV_CTRL_DISPLAY_VRR_MODE, &val);
+    if (ret != NvCtrlSuccess) {
+        return FALSE;
+    }
+
+    switch (val) {
+    case NV_CTRL_DISPLAY_VRR_MODE_GSYNC:
+        str = "G-SYNC";
+        break;
+    case NV_CTRL_DISPLAY_VRR_MODE_GSYNC_COMPATIBLE:
+        str = "G-SYNC Compatible";
+        break;
+    case NV_CTRL_DISPLAY_VRR_MODE_GSYNC_COMPATIBLE_UNVALIDATED:
+        str = "G-SYNC Unvalidated";
+        break;
+    default:
+    case NV_CTRL_DISPLAY_VRR_MODE_NONE:
+        str = "None";
+        break;
+    }
+
     gtk_label_set_text(GTK_LABEL(entry->txt), str);
-    g_free(str);
 
     return TRUE;
 }
 
 
 
+
+
 static gboolean update_refresh_rate(InfoEntry *entry)
 {
     CtkDisplayDevice *ctk_object = entry->ctk_object;
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
     ReturnStatus ret;
     gint val;
     gboolean hdmi3D;
     char *str;
     float fvalue;
 
-    ret = NvCtrlGetAttribute(ctk_object->handle, NV_CTRL_REFRESH_RATE, &val);
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_REFRESH_RATE, &val);
     if (ret != NvCtrlSuccess) {
         return FALSE;
     }
 
-    ret = NvCtrlGetAttribute(ctk_object->handle, NV_CTRL_DPY_HDMI_3D, &hdmi3D);
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_DPY_HDMI_3D, &hdmi3D);
     if (ret != NvCtrlSuccess) {
         return FALSE;
     }
@@ -771,7 +943,7 @@ static void update_device_info(CtkDisplayDevice *ctk_object)
 
         if (entry->present) {
             gtk_widget_show(entry->hbox);
-            gtk_widget_size_request(entry->label, &req);
+            ctk_widget_get_preferred_size(entry->label, &req);
             if (max_width < req.width) {
                 max_width = req.width;
             }
@@ -797,13 +969,14 @@ static void update_device_info(CtkDisplayDevice *ctk_object)
  */
 static void display_device_setup(CtkDisplayDevice *ctk_object)
 {
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
+
     /* Disable the reset button here and allow the controls below to (re)enable
      * it if need be,.
      */
     gtk_widget_set_sensitive(ctk_object->reset_button, FALSE);
 
-    update_display_enabled_flag(ctk_object->handle,
-                                &ctk_object->display_enabled);
+    update_display_enabled_flag(ctrl_target, &ctk_object->display_enabled);
 
     /* Update info */
 
@@ -887,7 +1060,8 @@ static gboolean unregister_refresh_rate_events(InfoEntry *entry)
  * this page should disable/enable access based on whether
  * or not the display device is enabled.
  */
-static void enabled_displays_received(GtkObject *object, gpointer arg1,
+static void enabled_displays_received(GObject *object,
+                                      CtrlEvent *event,
                                       gpointer user_data)
 {
     CtkDisplayDevice *ctk_object = CTK_DISPLAY_DEVICE(user_data);
@@ -899,7 +1073,8 @@ static void enabled_displays_received(GtkObject *object, gpointer arg1,
 } /* enabled_displays_received() */
 
 
-static void callback_link_changed(GtkObject *object, gpointer arg1,
+static void callback_link_changed(GObject *object,
+                                  CtrlEvent *event,
                                   gpointer user_data)
 {
     InfoEntry *entry = (InfoEntry *)user_data;
@@ -908,7 +1083,8 @@ static void callback_link_changed(GtkObject *object, gpointer arg1,
 }
 
 
-static void callback_refresh_rate_changed(GtkObject *object, gpointer arg1,
+static void callback_refresh_rate_changed(GObject *object,
+                                          CtrlEvent *event,
                                           gpointer user_data)
 {
     InfoEntry *entry = (InfoEntry *)user_data;
@@ -923,12 +1099,13 @@ static void add_color_correction_tab(CtkDisplayDevice *ctk_object,
                                      GtkWidget *notebook,
                                      ParsedAttribute *p)
 {
+    CtrlTarget *ctrl_target = ctk_object->ctrl_target;
     ReturnStatus ret;
     gint val;
     GtkWidget *ctk_color_correction;
     GtkWidget *box;
 
-    ret = NvCtrlGetAttribute(ctk_object->handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_ATTR_RANDR_GAMMA_AVAILABLE, &val);
 
     if (ret != NvCtrlSuccess) {
@@ -939,7 +1116,7 @@ static void add_color_correction_tab(CtkDisplayDevice *ctk_object,
         return;
     }
 
-    ctk_color_correction = ctk_color_correction_new(ctk_object->handle,
+    ctk_color_correction = ctk_color_correction_new(ctrl_target,
                                                     ctk_config,
                                                     p,
                                                     ctk_event);
@@ -954,4 +1131,5 @@ static void add_color_correction_tab(CtkDisplayDevice *ctk_object,
 
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box,
                              gtk_label_new("Color Correction"));
+    gtk_widget_show(box);
 }
