@@ -28,6 +28,7 @@
  *   NV-CONTROL -> event -> glib -> CtkEvent -> signal -> GUI
  */
 
+
 #include <string.h>
 
 #include <gtk/gtk.h>
@@ -331,7 +332,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_THERMAL_COOLER_SPEED);
     MAKE_SIGNAL(NV_CTRL_PALETTE_UPDATE_EVENT);
     MAKE_SIGNAL(NV_CTRL_VIDEO_ENCODER_UTILIZATION);
-    MAKE_SIGNAL(NV_CTRL_GSYNC_ALLOWED);
+    MAKE_SIGNAL(NV_CTRL_VRR_ALLOWED);
     MAKE_SIGNAL(NV_CTRL_GPU_NVCLOCK_OFFSET);
     MAKE_SIGNAL(NV_CTRL_GPU_MEM_TRANSFER_RATE_OFFSET);
     MAKE_SIGNAL(NV_CTRL_GPU_NVCLOCK_OFFSET_ALL_PERFORMANCE_LEVELS);
@@ -339,7 +340,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_VIDEO_DECODER_UTILIZATION);
     MAKE_SIGNAL(NV_CTRL_GPU_OVER_VOLTAGE_OFFSET);
     MAKE_SIGNAL(NV_CTRL_GPU_CURRENT_CORE_VOLTAGE);
-    MAKE_SIGNAL(NV_CTRL_SHOW_GSYNC_VISUAL_INDICATOR);
+    MAKE_SIGNAL(NV_CTRL_SHOW_VRR_VISUAL_INDICATOR);
     MAKE_SIGNAL(NV_CTRL_THERMAL_COOLER_CURRENT_LEVEL);
     MAKE_SIGNAL(NV_CTRL_STEREO_SWAP_MODE);
     MAKE_SIGNAL(NV_CTRL_GPU_FRAMELOCK_FIRMWARE_UNSUPPORTED);
@@ -347,6 +348,8 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_DISPLAYPORT_IS_MULTISTREAM);
     MAKE_SIGNAL(NV_CTRL_DISPLAYPORT_SINK_IS_AUDIO_CAPABLE);
     MAKE_SIGNAL(NV_CTRL_SHOW_GRAPHICS_VISUAL_INDICATOR);
+    MAKE_SIGNAL(NV_CTRL_DISPLAY_VRR_MODE);
+    MAKE_SIGNAL(NV_CTRL_DISPLAY_VRR_MIN_REFRESH_RATE);
 #undef MAKE_SIGNAL
 
     /*
@@ -356,7 +359,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
      * knows about.
      */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_SHOW_GRAPHICS_VISUAL_INDICATOR
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_DISPLAY_VRR_MIN_REFRESH_RATE
 #warning "There are attributes that do not emit signals!"
 #endif
 
